//
//  TestApp.m
//  PrefsPane
//
//  Created by Andreas on Sun Jun 08 2003.
//  Copyright (c) 2003 Andreas Mayer. All rights reserved.
//
//	2005-05-05	Andreas Mayer
//	- added menu item to open prefs pane with unified title and toolbar


#import "TestApp.h"
#import "TestPrefPane.h"


@implementation TestApp


- (void)dealloc
{
	[preferences release];
	[super dealloc];
}

- (IBAction)showWindow:(id)sender
{
	if (!prefsWindow) {
		// create the preference window controller
		// create style mask
		unsigned int styleMask = (NSTitledWindowMask|NSClosableWindowMask|NSMiniaturizableWindowMask);
		if ([sender tag] == 1) {
			styleMask |= NSUnifiedTitleAndToolbarWindowMask;
		}
		AMPreferenceWindowController *controller = [[AMPreferenceWindowController alloc] initWithAutosaveName:@"PreferencePanel" windowStyleMask:styleMask];
		[self setPrefsWindow:controller];
		[controller release];
		// we act as delegate ourselves
		[prefsWindow setDelegate:self];
		// load plugins from the bundle's standard plugins folder
		[prefsWindow addPluginsOfType:nil fromPath:[[NSBundle mainBundle] builtInPlugInsPath]];
		
		// creating some test preference panes
		id <AMPrefPaneProtocol> testPrefPane = [[[TestPrefPane alloc] initWithIdentifier:@"T-1" label:@"Test Pref1" category:@"Test"] autorelease];
		[prefsWindow addPane:testPrefPane withIdentifier:[testPrefPane identifier]];
		
		testPrefPane = [[[TestPrefPane alloc] initWithIdentifier:@"T-2" label:@"Test Pref2" category:@"Test"] autorelease];
		[prefsWindow addPane:testPrefPane withIdentifier:[testPrefPane identifier]];
		
		testPrefPane = [[[TestPrefPane alloc] initWithIdentifier:@"T-3" label:@"Test Pref3" category:@"Test"] autorelease];
		[prefsWindow addPane:testPrefPane withIdentifier:[testPrefPane identifier]];
		
		testPrefPane = [[[TestPrefPane alloc] initWithIdentifier:@"T-4" label:@"Test Pref4" category:@"Test"] autorelease];
		[prefsWindow addPane:testPrefPane withIdentifier:[testPrefPane identifier]];
		
		testPrefPane = [[[TestPrefPane alloc] initWithIdentifier:@"T-5" label:@"Test Pref5" category:@"Test"] autorelease];
		[prefsWindow addPane:testPrefPane withIdentifier:[testPrefPane identifier]];
		
		testPrefPane = [[[TestPrefPane alloc] initWithIdentifier:@"T-6" label:@"Test Pref6" category:@"Test2"] autorelease];
		[prefsWindow addPane:testPrefPane withIdentifier:[testPrefPane identifier]];
		
		testPrefPane = [[[TestPrefPane alloc] initWithIdentifier:@"T-7" label:@"Test Pref7" category:@"Test2"] autorelease];
		[prefsWindow addPane:testPrefPane withIdentifier:[testPrefPane identifier]];
			
		testPrefPane = [[[TestPrefPane alloc] initWithIdentifier:@"T-8" label:@"Test Pref8" category:@"Test2"] autorelease];
		[prefsWindow addPane:testPrefPane withIdentifier:[testPrefPane identifier]];
		
		// set up some configuration options
		[prefsWindow setUsesConfigurationPane:YES];
		[prefsWindow setSortByCategory:YES];
		// select prefs pane for display
		[prefsWindow selectPaneWithIdentifier:@"T-1"];
	}
	[prefsWindow showWindow:self];
	[[prefsWindow window] makeKeyAndOrderFront:self];
}

- (void)sortByAlphabet:(id)sender
{
	[prefsWindow setSortByCategory:NO];
	[prefsWindow selectIconViewPane];
}

- (void)sortByCategory:(id)sender
{
	[prefsWindow setSortByCategory:YES];
	[prefsWindow selectIconViewPane];
}


- (AMPreferenceWindowController *)prefsWindow
{
    return prefsWindow;
}

- (void)setPrefsWindow:(AMPreferenceWindowController *)newPrefsWindow
{
    id old = nil;

    if (newPrefsWindow != prefsWindow) {
        old = prefsWindow;
        prefsWindow = [newPrefsWindow retain];
        [old release];
    }
}

- (BOOL)shouldLoadPreferencePane:(NSString *)identifier
{
	NSLog(@"shouldLoadPreferencePane: %@", identifier);
	return YES;
}

- (void)willSelectPreferencePane:(NSString *)identifier
{
	NSLog(@"willSelectPreferencePane: %@", identifier);
}

- (void)didUnselectPreferencePane:(NSString *)identifier
{
	NSLog(@"didUnselectPreferencePane: %@", identifier);
}

- (NSString *)displayNameForCategory:(NSString *)category
{
	NSString *result = nil;
	if ([category isEqual:@"Test Category"]) {
		result = @"Allgemein";
	} else if ([category isEqual:@"Test"]) {
		result = @"Versuch";
	}
	return result;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	preferences = [[NSMutableDictionary alloc] init];
}


@end
