//
//  AMPreferenceWindowController.h
//  PrefPane
//
//  Created by Andreas on Sat May 31 2003.
//  Copyright (c) 2003 Andreas Mayer. All rights reserved.
//
//	2004-07-11	Andreas Mayer
//	- added -prefPaneWithIdentifier:
//	- added -didLoadPreferencePane: delegate method
//	2005-04-25	R-Project.org
//	- changed declaration of -addPane:withIdentifier:
//	2005-05-05	Andreas Mayer
//	- added -initWithAutosaveName:windowStyleMask:
//  2011-03-27 Andreas Mayer
//	- added precompiler conditionals for 10.6 SDK support


#import <Cocoa/Cocoa.h>
#import "AMPreferencePane.h"


#define AMPPToolbarIdentifier @"AMPPToolbar"
#define AMPPToolbarShowAllItemIdentifier @"AMPPToolbarShowAll"


#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1060

@interface AMPreferenceWindowController : NSWindowController <NSWindowDelegate, NSToolbarDelegate> {

#else

@interface AMPreferenceWindowController : NSWindowController {

#endif

	NSMutableDictionary *prefPanes;
	NSMutableArray *availablePlugins;
	NSView *iconView;
	AMPreferencePane *activePane;
	BOOL usesConfigurationPane;
	id delegate;
	NSString *autosaveName;
	NSString *title;
	BOOL _am_delegateRespondsToCategoryForPreferencePane;
	BOOL _am_delegateRespondsToDisplayNameForCategory;
}

- (id)initWithAutosaveName:(NSString *)name;

- (id)initWithAutosaveName:(NSString *)name windowStyleMask:(unsigned int)styleMask;

- (id)delegate;
- (void)setDelegate:(id)newDelegate;

- (BOOL)usesConfigurationPane;
- (void)setUsesConfigurationPane:(BOOL)newUsesConfigurationPane;
// needs to be set before actually showing the prefs window

- (BOOL)sortByCategory;
- (void)setSortByCategory:(BOOL)newSortByCategory;

- (NSArray *)categorySortOrder;
- (void)setCategorySortOrder:(NSArray *)newCategorySortOrder;

- (NSString *)title;
- (void)setTitle:(NSString *)newTitle;


- (BOOL)addPane:(id <AMPrefPaneProtocol>)newPane withIdentifier:(NSString *)identifier;

- (BOOL)addPluginFromPath:(NSString *)path;
// adds a plugin and loads it imediately

- (void)addPluginsOfType:(NSString *)extension fromPath:(NSString *)path;
// add plugins to available list
// if you do not require a special type, pass nil for pluginType
// (see NSBundle's pathForResource:ofType:)

- (void)loadPlugins;
// load plugins from available list depending on version number

- (BOOL)selectPaneWithIdentifier:(NSString *)identifier;

- (BOOL)selectIconViewPane;

- (void)replyToShouldUnselect:(BOOL)shouldUnselect;

- (AMPreferencePane *)prefPaneWithIdentifier:(NSString *)identifier;

- (NSDictionary *)prefPanes;

- (NSString *)categoryForIdentifier:(NSString *)identifier;

- (NSArray *)categories;

- (NSString *)categoryDisplayNameForIdentifier:(NSString *)identifier;

@end


@interface NSObject (AMPrefPaneDelegate)

- (NSString *)displayNameForCategory:(NSString *)category;

- (NSString *)categoryForPreferencePane:(NSString *)identifier defaultCategory:(NSString *)category;

- (BOOL)shouldLoadPreferencePane:(NSString *)identifier;
- (void)didLoadPreferencePane:(NSString *)identifier;
- (void)willSelectPreferencePane:(NSString *)identifier;
- (void)didUnselectPreferencePane:(NSString *)identifier;

@end

